# -*- coding: utf-8 -*-
"""
Created on Tue Jul  9 18:02:17 2024

@author: Arnd Helmut Hafner
"""

#語釈ファイルのファイル名および各ファイルに格納されている見出し語を読み取り、　
#indexdata.pyより出力された見出し語と比較する。

import sys 
import re
from pathlib import Path
import tools

def getgoshakumidashi(filename):
    """
    語釈ファイルを開き、　見出し語を取得する。

    Parameters
    ----------
    filename : str
        語釈ファイルのファイル名

    Returns
    -------
    midashi : str
    ID : str
       索引の種別

    """
    
    goshakufile = open(filename, 'r', encoding='utf-8-sig')
    for line in goshakufile:
        m = re.search('<title>語釈（([^）]+)）：(.+)</title>',line)
        #外字には 「<」　が含まれるから、　見出し語を取り出す正規表現としては[^<]+が仕えない
        if m: 
            return(m.group(1),m.group(2))
    sys.exit('Did not find midashi in {}'.format(filename))
            
def goshakumidashidic(filelist,logfile):
    """
    #語釈ファイルから見出し語を取得し、　見出し語をキーとする辞書として返す。

    Parameters
    ----------
    filelist : list
        語釈ファイルのパスとファイル名のリスト
    logfile : TestIOWrapper
        検証用のデータを蓄積していく

    Returns
    -------
    goshakumidashi : dic
        {見出し語:seiki,ID,filename}
    """
    goshakumidashi = {}
    for filename in filelist:
        ID, midashitmp = getgoshakumidashi(filename)
        if ID == '地名' and re.search('[郷県道亭郡津国関郵]',midashitmp):
            #logfile.write('重複地名：{}\n'.format(midashitmp))
            continue
        seiki = re.sub('（[^）]+）','', midashitmp)
        if midashitmp not in goshakumidashi.keys():
            goshakumidashi[midashitmp] = (seiki,ID,filename)
        else:
            midashitmp02 = midashitmp + '（' + goshakumidashi[midashitmp][1] + '）'
            goshakumidashi[midashitmp02] = (goshakumidashi[midashitmp][0],goshakumidashi[midashitmp][1],goshakumidashi[midashitmp][2])
            midashitmp02 = midashitmp + '（' + ID + '）'
            goshakumidashi[midashitmp02] = (seiki,ID,filename)
            del goshakumidashi[midashitmp]
    return(goshakumidashi)

def sakuinmidashidic(filename):    
    """
    #索引データの見出し語をファイルから読み込み、　見出し語をキーとする辞書として返す。

    Parameters
    ----------
    filename : str
        indexdata.pyから生成された見出し語一覧のテキストファイル

    Returns
    -------
    sakuinmidashi : dic
        {見出し語:seiki,ID,lemmatano}
    """

    sakuinmidashi = {}
    lemmatafile = open(filename, 'r', encoding='utf-8-sig')
    for line in lemmatafile:
        line = re.sub('\n','',line)
        lemmatano,midashitmp,seiki,ID = line.split('\t')
        if midashitmp not in sakuinmidashi.keys():
            sakuinmidashi[midashitmp] = (seiki,ID,lemmatano)
        else:
            midashitmp02 = midashitmp + '（' + sakuinmidashi[midashitmp][1] + '）'
            sakuinmidashi[midashitmp02] = (sakuinmidashi[midashitmp][0],sakuinmidashi[midashitmp][1],sakuinmidashi[midashitmp][2])
            midashitmp02 = midashitmp + '（' + ID + '）'
            sakuinmidashi[midashitmp02] = (seiki,ID,lemmatano)
            del sakuinmidashi[midashitmp]
    return(sakuinmidashi)

def shogo(dica,dicb):
    """
    二つの辞書のキーを比較し、　bにあってaにないキーを返す。

    Parameters
    ----------
    dica : dic
    dicb : dic

    Returns
    -------
    keturaku : list

    """
    
    keturaku = []
    for kii in sorted (dicb.keys()):
        if kii not in dica.keys():
            keturaku.append(kii)
    return(keturaku)

#語釈フォルダのファイル名を取得する
p = Path('.')
filelist = list(p.glob('Goshaku/**/*.html'))

#語釈ファイルから見出し語を取得し、　見出し語をキーとする辞書に格納する。 検証用に結果をログファイルに出力する。
logfile = open('log(goshakufile).txt','w', encoding='utf-8-sig')
goshakumidashi = goshakumidashidic(filelist,logfile)
for midashitmp in goshakumidashi.keys():
    logfile.write('{}\t{}\t{}\t{}\n'.format(midashitmp,goshakumidashi[midashitmp][0],goshakumidashi[midashitmp][1],goshakumidashi[midashitmp][2]))
logfile.close()

#索引データの見出し語を読み込む。　検証用に見出し語の通し番号順に出力する。
#検証の結果、　人名索引データ読み込みにおける[瘛]と[｛恝＋广｝（瘛）]の混同を原因とする誤認が一つ検出された。
sakuinmidashi = sakuinmidashidic('output(lemmata).txt')

lemmata = {}
for midashitmp in sakuinmidashi.keys():
    if sakuinmidashi[midashitmp][2] not in lemmata.keys():
        lemmata[sakuinmidashi[midashitmp][2]] = midashitmp
        
logfile = open('log(sakuin).txt','w', encoding='utf-8-sig')
for lemmatano in sorted(lemmata.keys(), key = lambda x:int(x)):
    midashitmp = lemmata[lemmatano]
    logfile.write('{}\t{}\t{}\t{}\n'.format(sakuinmidashi[midashitmp][2],midashitmp,sakuinmidashi[midashitmp][0],sakuinmidashi[midashitmp][1]))
logfile.close()

#尞辞書の照合
sakuinketuraku = shogo(sakuinmidashi,goshakumidashi)
tools.listoutput(sakuinketuraku,'out(sakuinketuraku).txt')
goshakuketuraku = shogo(goshakumidashi,sakuinmidashi)
tools.listoutput(goshakuketuraku,'out(goshakuketuraku).txt')




